@@ -79,7 +79,7 @@ class _ComboEntryPopup(gtk.Window):
         self._selection.set_mode(gtk.SELECTION_BROWSE)
         self._treeview.append_column(
             gtk.TreeViewColumn('Foo', gtk.CellRendererText(),
-                               text=0))
+                               markup=0))
         self._treeview.set_headers_visible(False)
         self._sw.add(self._treeview)
         self._treeview.show()
@@ -394,10 +394,7 @@ class ComboEntry(gtk.HBox):
         self._popup.set_size_request(-1, 24)
 
         completion = KiwiEntryCompletion()
-        completion.set_popup_window(self._popup)
-        completion.set_treeview(self._popup._treeview)
-        self.entry.set_completion(completion)
-        self.set_model(completion.get_model())
+        self.set_completion(completion)
 
     # Virtual methods
 
@@ -421,6 +418,8 @@ class ComboEntry(gtk.HBox):
 
     def _on_entry__scroll_event(self, entry, event):
         model = self.get_model()
+        if not len(model):
+            return
         treeiter = self._popup.get_selected_iter()
         # If nothing is selected, select the first one
         if not treeiter:
@@ -676,6 +675,12 @@ class ComboEntry(gtk.HBox):
 
         return items
 
+    def set_completion(self, completion):
+        completion.set_popup_window(self._popup)
+        completion.set_treeview(self._popup._treeview)
+        self.entry.set_completion(completion)
+        self.set_model(completion.get_model())
+
     # IconEntry
 
     def set_pixbuf(self, pixbuf):
