@@ -91,6 +91,10 @@ An enchanced version of GtkEntry that supports icons and masks
 
 import gettext
 import string
+try:
+    set
+except AttributeError:
+    from sets import Set as set
 
 import gobject
 import pango
@@ -634,11 +638,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
             return completion
 
         if old and isinstance(old, KiwiEntryCompletion):
-            if old.completion_timeout:
-                gobject.source_remove(old.completion_timeout)
-                old.completion_timeout = 0
-
-            old._disconnect_completion_signals()
+            old.disconnect_completion_signals()
 
         self._completion = completion
 
@@ -664,9 +664,11 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         model = completion.get_model()
         if not len(model):
             return
-
-        content = model[iter][COL_TEXT].lower()
-        return key.lower() in content
+        raw_content = model[iter][COL_TEXT]
+        if raw_content is not None:
+            return key.lower() in raw_content.lower()
+        else:
+            return False
 
     def _on_completion__match_selected(self, completion, model, iter):
         if not len(model):
@@ -989,7 +991,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         else:
             mode = self._mode
 
-        values = {}
+        values = set()
         if mode == ENTRY_MODE_TEXT:
             if sort:
                 itemdata.sort()
@@ -999,7 +1001,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
                     raise KeyError("Tried to insert duplicate value "
                                    "%r into the entry" % item)
                 else:
-                    values[item] = None
+                    values.add(item)
 
                 model.append((item, None))
         elif mode == ENTRY_MODE_DATA:
@@ -1008,11 +1010,14 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
             for item in itemdata:
                 text, data = item
-                if text in values:
-                    raise KeyError("Tried to insert duplicate value "
-                                   "%r into the entry" % text)
-                else:
-                    values[text] = None
+                # Add (n) to the end in case of duplicates
+                count = 1
+                orig = text
+                while text in values:
+                    text = orig + ' (%d)' % count
+                    count += 1
+
+                values.add(text)
                 model.append((text, data))
         else:
             raise TypeError("Incorrect format for itemdata; see "
