@@ -32,6 +32,11 @@ The GtkComboBox and GtkComboBoxEntry classes here are also slightly extended
 they contain methods to easily insert and retrieve data from combos.
 """
 
+try:
+    set
+except AttributeError:
+    from sets import Set as set
+
 import gobject
 import gtk
 from gtk import keysyms
@@ -80,7 +85,7 @@ class _EasyComboBoxHelper(object):
     def prefill(self, itemdata, sort=False):
         if not isinstance(itemdata, (list, tuple)):
             raise TypeError("'data' parameter must be a list or tuple of item "
-                            "descriptions, found %s") % type(itemdata)
+                            "descriptions, found %s" % (type(itemdata),))
 
         self.clear()
         if len(itemdata) == 0:
@@ -97,9 +102,13 @@ class _EasyComboBoxHelper(object):
                                 "be strings or tuple/list")
 
         mode = self.mode
+        if mode not in (ComboMode.STRING, ComboMode.DATA):
+            raise TypeError("Incorrect format for itemdata; see "
+                            "docstring for more information")
+
         model = self._combobox.get_model()
 
-        values = {}
+        values = set()
         if mode == ComboMode.STRING:
             if sort:
                 itemdata.sort()
@@ -107,9 +116,8 @@ class _EasyComboBoxHelper(object):
             for item in itemdata:
                 if item in values:
                     raise KeyError("Tried to insert duplicate value "
-                                   "%s into Combo!" % item)
-                else:
-                    values[item] = None
+                                   "%s into Combo!" % (item,))
+                values.add(item)
 
                 model.append((item, None))
         elif mode == ComboMode.DATA:
@@ -118,15 +126,13 @@ class _EasyComboBoxHelper(object):
 
             for item in itemdata:
                 text, data = item
-                if text in values:
-                    raise KeyError("Tried to insert duplicate value "
-                                   "%s into Combo!" % item)
-                else:
-                    values[text] = None
+                orig = text
+                count = 1
+                while text in values:
+                    text = orig + ' (%d)' % count
+                    count += 1
+                values.add(text)
                 model.append((text, data))
-        else:
-            raise TypeError("Incorrect format for itemdata; see "
-                            "docstring for more information")
 
     def append_item(self, label, data=None):
         """ Adds a single item to the Combo. Takes:
@@ -134,7 +140,7 @@ class _EasyComboBoxHelper(object):
         - data: the data to be associated with that item
         """
         if not isinstance(label, basestring):
-            raise TypeError("label must be string, found %s" % label)
+            raise TypeError("label must be string, found %s" % (label,))
 
         if self.mode == ComboMode.UNKNOWN:
             if data is not None:
@@ -154,6 +160,33 @@ class _EasyComboBoxHelper(object):
         else:
             raise AssertionError
 
+    def insert_item(self, position, label, data=None):
+        """ Inserts a single item at a position to the Combo.
+        @param position: position to insert the item at
+        @param label: a string with the text to be added
+        @param data: the data to be associated with that item
+        """
+        if not isinstance(label, basestring):
+            raise TypeError("label must be string, found %s" % (label,))
+
+        if self.mode == ComboMode.UNKNOWN:
+            if data is not None:
+                self.set_mode(ComboMode.DATA)
+            else:
+                self.set_mode(ComboMode.STRING)
+
+        model = self._combobox.get_model()
+        if self.mode == ComboMode.STRING:
+            if data is not None:
+                raise TypeError("data can not be specified in string mode")
+            model.insert(position, (label, None))
+        elif self.mode == ComboMode.DATA:
+            if data is None:
+                raise TypeError("data must be specified in string mode")
+            model.insert(position, (label, data))
+        else:
+            raise AssertionError
+
     def select(self, data):
         mode = self.mode
         if self.mode == ComboMode.STRING:
@@ -279,7 +312,7 @@ class ProxyComboBox(PropertyObject, gtk.ComboBox, ProxyWidgetMixin):
         # We dont need validation because the user always
         # choose a valid value
 
-        if data is None:
+        if data is None or data is ValueUnset:
             return
 
         if self._helper.get_mode() == ComboMode.STRING:
@@ -312,6 +345,12 @@ class ProxyComboBox(PropertyObject, gtk.ComboBox, ProxyWidgetMixin):
         """
         self._helper.append_item(label, data)
 
+    def insert_item(self, position, label, data=None):
+        """
+        See L{kiwi.interfaces.IEasyCombo.insert_item}
+        """
+        self._helper.insert_item(position, label, data)
+
     def select(self, data):
         """
         See L{kiwi.interfaces.IEasyCombo.select}
@@ -474,7 +513,7 @@ class ProxyComboBoxEntry(PropertyObject, BaseComboBoxEntry,
 
     # IEasyCombo
 
-    def prefill(self, itemdata, sort=False, clear_entry=False):
+    def prefill(self, itemdata, sort=False, clear_entry=True):
         """
         See L{kiwi.interfaces.IEasyCombo.prefill}
         """
@@ -505,6 +544,12 @@ class ProxyComboBoxEntry(PropertyObject, BaseComboBoxEntry,
         """
         self._helper.append_item(label, data)
 
+    def insert_item(self, position, label, data=None):
+        """
+        See L{kiwi.interfaces.IEasyCombo.insert_item}
+        """
+        self._helper.insert_item(position, label, data)
+
     def select(self, data):
         """
         See L{kiwi.interfaces.IEasyCombo.select}
